/**********************************************************************
 *
 * gsdlconf.h -- system specific includes
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * A component of the Greenstone digital library software
 * from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: gsdlconf.h 595 1999-09-16 21:38:17Z sjboddie $
 *
 *********************************************************************/


#ifndef GSDLCONF_H
#define GSDLCONF_H


// Note: all macros used in the gsdl software (except the _H macros) should
// now start with GSDL_


// GSDL_NEED_STRINGS_H should be defined if some string functions
// needed (like strcasecmp) are in strings.h
#if defined(__IBMCPP__)
#  define GSDL_NEED_STRINGS_H
#endif

// GSDL_USE_IOS_H should be defined if the .h extension needs to
// be used for the io stream headers
#if defined(__GNUG__) || defined(__IBMCPP__)
#  define GSDL_USE_IOS_H 1
#endif

// GSDL_USE_STL_H should be defined if the .h extension needs to
// be used for the stl header files
#if defined(__IBMCPP__)
#  define GSDL_USE_STL_H 1
#endif

// GSDL_USE_ALGO_H should be defined if algo.h should be used 
// instead of algorithm.h
#if defined(__IBMCPP__)
#  define GSDL_USE_ALGO_H 1
#endif

// GSDL_NAMESPACE_BROKEN should be defined if namespaces should
// not be used
#if defined(__GNUG__) || defined(__IBMCPP__)
#  define GSDL_NAMESPACE_BROKEN 1
#endif

// GSDL_USE_OBJECTSPACE should now be used instead of USE_OBJECTSPACE
#if defined(USE_OBJECTSPACE)
#  define GSDL_USE_OBJECTSPACE
#endif

// GSDL_NEED_DESTROY_USHORT should be defined if the compiler needs
// a definition for void destroy(short unsigned int *)
#if defined(__IBMCPP__)
# define GSDL_NEED_DESTROY_USHORT
#endif


// file locking

#if defined(__WIN32__)
#define MY_UNLOCK    0
#define MY_READLOCK  1
#define MY_WRITELOCK 2
int my_locking(int fd, int lmode, long lockoffset, long nbytes);

#define UNLOCK_FILE(fd)    my_locking(fd, MY_UNLOCK, 0, 10)
#define READLOCK_FILE(fd)  lock_val = my_locking(fd, MY_READLOCK, 0, 10)
#define WRITELOCK_FILE(fd) lock_val = my_locking(fd, MY_WRITELOCK, 0, 10)

#else

#if GSDL_USE_FLOCK
#include <sys/lock.h>

#ifndef LOCK_SH
#define LOCK_SH	1
#endif

#ifndef LOCK_EX
#define LOCK_EX	2
#endif

#ifndef LOCK_NB
#define LOCK_NB 4
#endif

#ifndef LOCK_UN
#define LOCK_UN 8
#endif

#define UNLOCK_FILE(fd) flock (fd, LOCK_UN)
#define READLOCK_FILE(fd) lock_val = flock (fd, LOCK_SH + LOCK_NB)
#define WRITELOCK_FILE(fd) lock_val = flock (fd, LOCK_EX + LOCK_NB)

#else
// use fcntl
#include <fcntl.h>
#define UNLOCK_FILE(fd) \
	{					\
	  struct flock flock;			\
	  flock.l_type = F_UNLCK;		\
	  flock.l_whence = SEEK_SET;		\
	  flock.l_start = flock.l_len = 0L;	\
	  fcntl (fd, F_SETLK, &flock);	\
	}
#define READLOCK_FILE(fd) \
	{					\
	  struct flock flock;			\
	  flock.l_type = F_RDLCK;		\
	  flock.l_whence = SEEK_SET;			\
	  flock.l_start = flock.l_len = 0L;	\
	  lock_val = fcntl (fd, F_SETLK, &flock);	\
	}
#define WRITELOCK_FILE(fd) \
	{					\
	  struct flock flock;			\
	  flock.l_type = F_WRLCK;		\
	  flock.l_whence = SEEK_SET;			\
	  flock.l_start = flock.l_len = 0L;	\
	  lock_val = fcntl (fd, F_SETLK, &flock);	\
	}

#endif
#endif

#endif
